import { Component, Input, ContentChildren, ViewChild, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class BrushComponent {
    constructor() {
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (this.signalNotifier) {
            this.signalNotifier.notify();
        }
    }
}
BrushComponent.decorators = [
    { type: Component, args: [{
                selector: 'highlight-brush',
                template: ''
            }] }
];
/** @nocollapse */
BrushComponent.ctorParameters = () => [];
BrushComponent.propDecorators = {
    brushClass: [{ type: Input }],
    lineWidth: [{ type: Input }],
    lineCap: [{ type: Input }],
    lineJoin: [{ type: Input }],
    strokeStyle: [{ type: Input }],
    lineDash: [{ type: Input }],
    fillStyle: [{ type: Input }]
};
class BrushWithDefaults {
    /**
     * @param {?} brush
     */
    constructor(brush) {
        this.brush = brush;
    }
    /**
     * @return {?}
     */
    lineDash() {
        /** @type {?} */
        const lineDash = this.getValue('lineDash', [0]);
        if (typeof lineDash === 'string') {
            return lineDash.split(',').map((/**
             * @param {?} value
             * @return {?}
             */
            value => parseInt(value.trim(), 10)));
        }
        else {
            return lineDash;
        }
    }
    /**
     * @return {?}
     */
    lineCap() {
        return this.getValue('lineCap', 'round');
    }
    /**
     * @return {?}
     */
    lineWidth() {
        return this.getValue('lineWidth', 1);
    }
    /**
     * @return {?}
     */
    lineJoin() {
        return this.getValue('lineJoin', 'round');
    }
    /**
     * @return {?}
     */
    strokeStyle() {
        return this.getValue('strokeStyle', '#FFFFFF00');
    }
    /**
     * @return {?}
     */
    fillStyle() {
        return this.getValue('fillStyle', '#FFFFFF00');
    }
    /**
     * @template T
     * @param {?} field
     * @param {?} defaultValue
     * @return {?}
     */
    getValue(field, defaultValue) {
        if (this.brush && this.brush[field]) {
            return this.brush[field];
        }
        return defaultValue;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class AreaComponent {
    constructor() {
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (this.signalNotifier && changes.brushClass) {
            this.signalNotifier.notify();
        }
    }
}
AreaComponent.decorators = [
    { type: Component, args: [{
                selector: 'highlight-area',
                template: "<area *ngIf=\"!inactive\" [href]=\"href\" shape=\"poly\" [coords]=\"coords\" [title]=\"title\" [alt]=\"alt\"/>\r\n",
                styles: [""]
            }] }
];
/** @nocollapse */
AreaComponent.ctorParameters = () => [];
AreaComponent.propDecorators = {
    href: [{ type: Input }],
    brushClass: [{ type: Input }],
    title: [{ type: Input }],
    alt: [{ type: Input }],
    coords: [{ type: Input }],
    inactive: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class SignalNotifier {
    /**
     * @param {?} delay
     * @param {?} handler
     */
    constructor(delay, handler) {
        this.delay = delay;
        this.handler = handler;
    }
    /**
     * @return {?}
     */
    notify() {
        if (this.timer) {
            window.clearTimeout(this.timer);
        }
        this.timer = window.setTimeout((/**
         * @return {?}
         */
        () => {
            this.run();
        }), this.delay);
    }
    /**
     * @private
     * @return {?}
     */
    run() {
        this.timer = null;
        this.handler();
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MapComponent {
    constructor() {
        this.brushesMap = {};
        this.signalNotifier = new SignalNotifier(100, this.repaint.bind(this));
    }
    /**
     * @return {?}
     */
    ngAfterViewInit() {
        this.context = this.canvasMap.nativeElement.getContext('2d');
        this.mainContainer.nativeElement.style.backgroundImage = `url(${this.src})`;
        this.prepareBrushesMap();
        this.reorderLayers();
        this.areas.forEach((/**
         * @param {?} area
         * @return {?}
         */
        area => {
            area.signalNotifier = this.signalNotifier;
        }));
        this.brushes.forEach((/**
         * @param {?} brush
         * @return {?}
         */
        brush => {
            brush.signalNotifier = this.signalNotifier;
        }));
    }
    /**
     * @return {?}
     */
    mapOut() {
        this.repaint();
        this.reorderLayers();
    }
    /**
     * @param {?} event
     * @return {?}
     */
    mapOver(event) {
        this.repaint();
        /** @type {?} */
        const area = this.findAreaByCoords(event.target.coords);
        /** @type {?} */
        const brush = this.findHoverBrush(area);
        this.initContext(brush);
        this.drawCoords(event.target.coords.split(','));
        this.reorderLayers();
    }
    /**
     * @return {?}
     */
    repaint() {
        if (this.canvasMap !== null) {
            this.mainContainer.nativeElement.style.width = `${this.highlightedImage.nativeElement.width}px`;
            this.mainContainer.nativeElement.style.height = `${this.highlightedImage.nativeElement.height}px`;
            this.canvasMap.nativeElement.width = this.highlightedImage.nativeElement.width;
            this.canvasMap.nativeElement.height = this.highlightedImage.nativeElement.height;
        }
        this.drawInitialAreas();
    }
    /**
     * @private
     * @return {?}
     */
    drawInitialAreas() {
        this.areas.forEach((/**
         * @param {?} area
         * @return {?}
         */
        area => {
            /** @type {?} */
            const brush = this.brushesMap[area.brushClass];
            if (brush) {
                this.initContext(new BrushWithDefaults(brush));
                this.drawCoords(area.coords.split(',').map((/**
                 * @param {?} value
                 * @return {?}
                 */
                value => parseInt(value, 10))));
            }
        }));
    }
    /**
     * @private
     * @param {?} area
     * @return {?}
     */
    findHoverBrush(area) {
        /** @type {?} */
        let brush;
        /** @type {?} */
        const hoverAttribute = ':hover';
        if (area !== undefined && this.brushesMap[area.brushClass + hoverAttribute] !== undefined) {
            brush = new BrushWithDefaults(this.brushesMap[area.brushClass + hoverAttribute]);
        }
        else {
            brush = new BrushWithDefaults(this.brushesMap[hoverAttribute]);
        }
        return brush;
    }
    /**
     * @private
     * @param {?} brush
     * @return {?}
     */
    initContext(brush) {
        this.context.setLineDash(brush.lineDash());
        this.context.strokeStyle = brush.strokeStyle();
        this.context.fillStyle = brush.fillStyle();
        this.context.lineCap = brush.lineCap();
        this.context.lineWidth = brush.lineWidth();
        this.context.lineJoin = brush.lineJoin();
    }
    /**
     * @private
     * @param {?} coords
     * @return {?}
     */
    drawCoords(coords) {
        this.context.beginPath();
        /** @type {?} */
        let i = 0;
        while (i < coords.length) {
            this.context.lineTo(coords[i], coords[i + 1]);
            i += 2;
        }
        this.context.closePath();
        this.context.stroke();
        this.context.fill();
    }
    /**
     * @private
     * @return {?}
     */
    reorderLayers() {
        this.highlightedImage.nativeElement.before(this.canvasMap.nativeElement);
    }
    /**
     * @private
     * @param {?} coords
     * @return {?}
     */
    findAreaByCoords(coords) {
        return this.areas.find((/**
         * @param {?} area
         * @return {?}
         */
        area => area.coords === coords));
    }
    /**
     * @private
     * @return {?}
     */
    prepareBrushesMap() {
        this.brushes.forEach((/**
         * @param {?} item
         * @return {?}
         */
        item => this.brushesMap[item.brushClass] = item));
    }
}
MapComponent.decorators = [
    { type: Component, args: [{
                selector: 'highlight-map',
                template: "<div class=\"highlight-container\" #mainContainer >\r\n  <img [src]=\"src\" useMap=\"#{{name}}\" class=\"highlight-image\" (load)=\"repaint()\" #highlightedImage>\r\n  <canvas class=\"highlight-canvas\" #canvasMap></canvas>\r\n  <map [name]=\"name\" (mouseover)=\"mapOver($event)\" (mouseout)=\"mapOut()\">\r\n    <ng-content></ng-content>\r\n  </map>\r\n</div>\r\n",
                styles: ["div.highlight-container{position:relative;background-repeat:no-repeat;padding:0;text-align:left}div.highlight-container .highlight-image{opacity:.000001}div.highlight-container .highlight-canvas{position:absolute;top:0;left:0}"]
            }] }
];
/** @nocollapse */
MapComponent.ctorParameters = () => [];
MapComponent.propDecorators = {
    mainContainer: [{ type: ViewChild, args: ['mainContainer',] }],
    canvasMap: [{ type: ViewChild, args: ['canvasMap',] }],
    highlightedImage: [{ type: ViewChild, args: ['highlightedImage',] }],
    areas: [{ type: ContentChildren, args: [AreaComponent,] }],
    brushes: [{ type: ContentChildren, args: [BrushComponent,] }],
    src: [{ type: Input }],
    name: [{ type: Input }]
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class MapModule {
}
MapModule.decorators = [
    { type: NgModule, args: [{
                declarations: [MapComponent, AreaComponent, BrushComponent],
                imports: [
                    CommonModule
                ],
                exports: [MapComponent, AreaComponent, BrushComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { MapComponent, MapModule, AreaComponent as ɵa, BrushComponent as ɵb };

//# sourceMappingURL=marciniak-map.js.map